#
# Copyright The NOMAD Authors.
#
# This file is part of NOMAD. See https://nomad-lab.eu for further info.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

import pytest
import numpy as np

from nomad.datamodel import EntryArchive
from atomisticparsers.amber import AmberParser


def approx(value, abs=0, rel=1e-6):
    return pytest.approx(value, abs=abs, rel=rel)


@pytest.fixture(scope='module')
def parser():
    return AmberParser()


def test_basic(parser):
    archive = EntryArchive()
    parser.parse('tests/data/amber/polyAT_vac_init_min.out', archive, None)

    assert archive.run[0].program.version == '14 SANDER 2014'
    sec_system = archive.run[0].system[0]
    assert np.shape(sec_system.atoms.positions) == (638, 3)
    assert sec_system.atoms.labels[7] == 'O'
    sec_scc = archive.run[0].calculation
    assert sec_scc[1].energy.total.value.magnitude == approx(-3.49178376e-16)
