#
# Copyright The NOMAD Authors.
#
# This file is part of NOMAD. See https://nomad-lab.eu for further info.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

import pytest

from nomad.datamodel import EntryArchive
from atomisticparsers.gromos import GromosParser


def approx(value, abs=0, rel=1e-6):
    return pytest.approx(value, abs=abs, rel=rel)


@pytest.fixture(scope='module')
def parser():
    return GromosParser()


def test_basic(parser):
    archive = EntryArchive()

    parser.parse('tests/data/gromos/eq/eq_peptide_1.omd', archive, None)

    sec_run = archive.run[0]
    assert sec_run.program.version == '1.3.1'

    sec_system = archive.run[0].system[0]
    assert sec_system.atoms.labels[10] == 'N'
    assert sec_system.atoms.positions[37][2].magnitude == approx(-1.0911137e-11)

    sec_sccs = sec_run.calculation
    assert len(sec_sccs) == 100
    assert sec_sccs[19].energy.total.value.magnitude == approx(-7.70938473e-17)
